// Exp.java                                         MM 2013

/**
 * Exponentialfunktion f(x) = e ^ x, 
 * D = R, W = ] 0, +unendlich[.
 * @author Monika Meiler
 */
public class Exp extends DifferenzierbareFunktion
{
/* ----------------------------------------------------- */
                                        // service-Methoden
/**
 * Berechnen eines Funktionswertes e ^ x.
 * @param arg Argument
 * @return e^arg
 */
  public double wert( double arg)
  {
// Trivialfaelle
    if( arg == 0) return 1;

// einfache Reihenentwicklung
//  return myExp( arg);
// verbesserte Reihenentwicklung
    return myExpBesser( arg);
  }

/**
 * Berechnen eines Funktionswertes durch Reihenentwicklung:
 * e ^ x = 1 + x + ( x ^ 2) / 2! + ( x ^ 3) / 3! ...
 * @param arg Argument
 * @return e^arg
 */
  private double myExp( double arg)
  {
    double x = arg;
    double alteSumme, neueSumme = 1, summand = 1;
    int i = 0;
    do                                 // Reihenentwicklung
    {
//    System.out.println
//    ( i + ": " + neueSumme + " " + summand);
      summand *= x / ++i;                  // neuer Summand

      alteSumme = neueSumme;                  // neue Summe
      neueSumme += summand;
    } while(( neueSumme != alteSumme) && ( i < 1000));

    return alteSumme;
  }

/**
 * Verbesserte Exponentialfunktion:
 * Fuer x < 0 berechne e^x = 1 / (e^-x).
 * @param arg Argument
 * @return e^arg verbessert
 */
  private double myExpBesser( double arg)
  {
    if( arg < 0) return 1 / myExp( -arg);
    return myExp( arg);
  }

/**
 * Berechnen der ersten Ableitung f'( x) = e ^ x.
 * @param arg Argument
 * @return f'( arg)
 */
  public double wertErsteAbleitung( double arg)
  {
    return wert( arg);
  }

/* ----------------------------------------------------- */
                                        // toString-Methode
/**
 * Darstellen der Exponentialfunktion.
 * @return Funktion in linearer Schreibweise
 */
  public String toString()
  {
    return "e ^ x";
  }
}